
package org.mule.modules.blogger.config;

import org.apache.commons.pool.KeyedPoolableObjectFactory;
import org.apache.commons.pool.impl.GenericKeyedObjectPool;
import org.mule.api.Capabilities;
import org.mule.api.Capability;
import org.mule.api.ConnectionManager;
import org.mule.api.MuleContext;
import org.mule.api.construct.FlowConstruct;
import org.mule.api.lifecycle.Disposable;
import org.mule.api.lifecycle.Initialisable;
import org.mule.api.lifecycle.Startable;
import org.mule.api.lifecycle.Stoppable;
import org.mule.config.PoolingProfile;
import org.mule.modules.blogger.BloggerConnector;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * A {@code BloggerConnectorConnectionManager} is a wrapper around {@link BloggerConnector } that adds connection management capabilities to the pojo.
 * 
 */
public class BloggerConnectorConnectionManager
    implements Capabilities, ConnectionManager<BloggerConnectorConnectionManager.ConnectionParameters, BloggerConnectorLifecycleAdapter> , Initialisable
{

    private String username;
    private String password;
    private String blogId;
    private static Logger logger = LoggerFactory.getLogger(BloggerConnectorConnectionManager.class);
    /**
     * Mule Context
     * 
     */
    private MuleContext muleContext;
    /**
     * Flow construct
     * 
     */
    private FlowConstruct flowConstruct;
    /**
     * Connector Pool
     * 
     */
    private GenericKeyedObjectPool connectionPool;
    protected PoolingProfile connectionPoolingProfile;

    /**
     * Sets blogId
     * 
     * @param value Value to set
     */
    public void setBlogId(String value) {
        this.blogId = value;
    }

    /**
     * Retrieves blogId
     * 
     */
    public String getBlogId() {
        return this.blogId;
    }

    /**
     * Sets connectionPoolingProfile
     * 
     * @param value Value to set
     */
    public void setConnectionPoolingProfile(PoolingProfile value) {
        this.connectionPoolingProfile = value;
    }

    /**
     * Retrieves connectionPoolingProfile
     * 
     */
    public PoolingProfile getConnectionPoolingProfile() {
        return this.connectionPoolingProfile;
    }

    /**
     * Sets username
     * 
     * @param value Value to set
     */
    public void setUsername(String value) {
        this.username = value;
    }

    /**
     * Retrieves username
     * 
     */
    public String getUsername() {
        return this.username;
    }

    /**
     * Sets password
     * 
     * @param value Value to set
     */
    public void setPassword(String value) {
        this.password = value;
    }

    /**
     * Retrieves password
     * 
     */
    public String getPassword() {
        return this.password;
    }

    /**
     * Sets flow construct
     * 
     * @param flowConstruct Flow construct to set
     */
    public void setFlowConstruct(FlowConstruct flowConstruct) {
        this.flowConstruct = flowConstruct;
    }

    /**
     * Set the Mule context
     * 
     * @param context Mule context to set
     */
    public void setMuleContext(MuleContext context) {
        this.muleContext = context;
    }

    public void initialise() {
        GenericKeyedObjectPool.Config config = new GenericKeyedObjectPool.Config();
        if (connectionPoolingProfile!= null) {
            config.maxIdle = connectionPoolingProfile.getMaxIdle();
            config.maxActive = connectionPoolingProfile.getMaxActive();
            config.maxWait = connectionPoolingProfile.getMaxWait();
            config.whenExhaustedAction = ((byte) connectionPoolingProfile.getExhaustedAction());
        }
        connectionPool = new GenericKeyedObjectPool(new BloggerConnectorConnectionManager.ConnectionFactory(this), config);
    }

    public BloggerConnectorLifecycleAdapter acquireConnection(BloggerConnectorConnectionManager.ConnectionParameters key)
        throws Exception
    {
        return ((BloggerConnectorLifecycleAdapter) connectionPool.borrowObject(key));
    }

    public void releaseConnection(BloggerConnectorConnectionManager.ConnectionParameters key, BloggerConnectorLifecycleAdapter connection)
        throws Exception
    {
        connectionPool.returnObject(key, connection);
    }

    public void destroyConnection(BloggerConnectorConnectionManager.ConnectionParameters key, BloggerConnectorLifecycleAdapter connection)
        throws Exception
    {
        connectionPool.invalidateObject(key, connection);
    }

    /**
     * Returns true if this module implements such capability
     * 
     */
    public boolean isCapableOf(Capability capability) {
        if (capability == Capability.LIFECYCLE_CAPABLE) {
            return true;
        }
        if (capability == Capability.CONNECTION_MANAGEMENT_CAPABLE) {
            return true;
        }
        return false;
    }

    private static class ConnectionFactory
        implements KeyedPoolableObjectFactory
    {

        private BloggerConnectorConnectionManager connectionManager;

        public ConnectionFactory(BloggerConnectorConnectionManager connectionManager) {
            this.connectionManager = connectionManager;
        }

        public Object makeObject(Object key)
            throws Exception
        {
            if (!(key instanceof BloggerConnectorConnectionManager.ConnectionParameters)) {
                throw new RuntimeException("Invalid key type");
            }
            BloggerConnectorLifecycleAdapter connector = new BloggerConnectorLifecycleAdapter();
            connector.setBlogId(connectionManager.getBlogId());
            if (connector instanceof Initialisable) {
                connector.initialise();
            }
            if (connector instanceof Startable) {
                connector.start();
            }
            return connector;
        }

        public void destroyObject(Object key, Object obj)
            throws Exception
        {
            if (!(key instanceof BloggerConnectorConnectionManager.ConnectionParameters)) {
                throw new RuntimeException("Invalid key type");
            }
            if (!(obj instanceof BloggerConnectorLifecycleAdapter)) {
                throw new RuntimeException("Invalid connector type");
            }
            try {
                ((BloggerConnectorLifecycleAdapter) obj).disconnect();
            } catch (Exception e) {
                throw e;
            } finally {
                if (((BloggerConnectorLifecycleAdapter) obj) instanceof Stoppable) {
                    ((BloggerConnectorLifecycleAdapter) obj).stop();
                }
                if (((BloggerConnectorLifecycleAdapter) obj) instanceof Disposable) {
                    ((BloggerConnectorLifecycleAdapter) obj).dispose();
                }
            }
        }

        public boolean validateObject(Object key, Object obj) {
            if (!(obj instanceof BloggerConnectorLifecycleAdapter)) {
                throw new RuntimeException("Invalid connector type");
            }
            try {
                return ((BloggerConnectorLifecycleAdapter) obj).isConnected();
            } catch (Exception e) {
                logger.error(e.getMessage(), e);
                return false;
            }
        }

        public void activateObject(Object key, Object obj)
            throws Exception
        {
            if (!(key instanceof BloggerConnectorConnectionManager.ConnectionParameters)) {
                throw new RuntimeException("Invalid key type");
            }
            if (!(obj instanceof BloggerConnectorLifecycleAdapter)) {
                throw new RuntimeException("Invalid connector type");
            }
            try {
                if (!((BloggerConnectorLifecycleAdapter) obj).isConnected()) {
                    ((BloggerConnectorLifecycleAdapter) obj).connect(((BloggerConnectorConnectionManager.ConnectionParameters) key).getUsername(), ((BloggerConnectorConnectionManager.ConnectionParameters) key).getPassword());
                }
            } catch (Exception e) {
                throw e;
            }
        }

        public void passivateObject(Object key, Object obj)
            throws Exception
        {
        }

    }

    public static class ConnectionParameters {

        private String username;
        private String password;

        public ConnectionParameters(String username, String password) {
            this.username = username;
            this.password = password;
        }

        /**
         * Sets username
         * 
         * @param value Value to set
         */
        public void setUsername(String value) {
            this.username = value;
        }

        /**
         * Retrieves username
         * 
         */
        public String getUsername() {
            return this.username;
        }

        /**
         * Sets password
         * 
         * @param value Value to set
         */
        public void setPassword(String value) {
            this.password = value;
        }

        /**
         * Retrieves password
         * 
         */
        public String getPassword() {
            return this.password;
        }

        public int hashCode() {
            int hash = 1;
            hash = ((hash* 31)+ this.username.hashCode());
            return hash;
        }

        public boolean equals(Object obj) {
            return ((obj instanceof BloggerConnectorConnectionManager.ConnectionParameters)&&(this.username == ((BloggerConnectorConnectionManager.ConnectionParameters) obj).username));
        }

    }

}
